"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("source-map-support/register");
const path_1 = require("path");
const typescript_transformer_common_1 = require("@build-script/typescript-transformer-common");
const node_1 = require("@idlebox/node");
const fancy_log_1 = require("fancy-log");
const fs_extra_1 = require("fs-extra");
const typescript_1 = require("typescript");
/** @external */
exports.default = typescript_transformer_common_1.createProgramPlugin(function plugin(program, { verbose } = {}, { rootDir, outDir, packageJsonPath }) {
    const debug = typescript_transformer_common_1.getDebug(verbose || false);
    debug('new program... %s files', program.getSourceFiles().length);
    const wrapImportFileName = function (f) {
        return f.replace(rootDir, outDir).replace(/\.tsx?$/, '.js') + '.importinfo';
    };
    debug('load package.json: %s', packageJsonPath);
    const dependencies = typescript_transformer_common_1.createDependencies(packageJsonPath);
    debug('    dependencies count: %s', Object.keys(dependencies).length);
    return function transformer(transformationContext) {
        transformationContext.enableEmitNotification(typescript_1.SyntaxKind.SourceFile);
        return (sourceFile) => {
            const { fileName } = sourceFile;
            const importInfoFile = { externals: [], internals: [], errors: [] };
            debug('[trans] visit \x1B[2m%s\x1B[0m', fileName);
            const result = typescript_1.visitEachChild(sourceFile, visitNode, transformationContext);
            debug('[trans] visit complete');
            transformationContext.onEmitNode(typescript_1.EmitHint.SourceFile, sourceFile, () => {
                const distFile = wrapImportFileName(fileName);
                // console.log('---------------', distFile, importInfoFile);
                fs_extra_1.mkdirpSync(path_1.dirname(distFile));
                if (importInfoFile.errors.length === 0) {
                    delete importInfoFile.errors;
                }
                node_1.writeFileIfChangeSync(distFile, JSON.stringify(importInfoFile, null, 4));
            });
            return result;
            function visitNode(node) {
                try {
                    if (typescript_transformer_common_1.isImportExport(node)) {
                        const importId = typescript_transformer_common_1.getImportedName(node);
                        if (typescript_transformer_common_1.isImportNative(importId)) {
                            return;
                        }
                        debug(' * %s', node.getText().split('\n').join(' '));
                        if (typescript_transformer_common_1.isImportFromNodeModules(importId)) {
                            const { packageName } = typescript_transformer_common_1.splitPackageName(importId);
                            if (!dependencies[packageName]) {
                                importInfoFile.errors.push(typescript_transformer_common_1.missing(importId));
                                fancy_log_1.error(typescript_transformer_common_1.createDiagnosticMissingImport(node));
                            }
                            else {
                                const info = typescript_transformer_common_1.resolveTypescriptModule(node, packageJsonPath);
                                if (info.type === 'missing') {
                                    fancy_log_1.error(typescript_transformer_common_1.createDiagnosticMissingImport(node));
                                    importInfoFile.errors.push(info);
                                }
                                else {
                                    importInfoFile.externals.push(info);
                                }
                            }
                        }
                        else {
                            const info = typescript_transformer_common_1.resolveProjectFile(node, program);
                            if (info.type === 'missing') {
                                fancy_log_1.error(typescript_transformer_common_1.createDiagnosticMissingImport(node));
                                importInfoFile.errors.push(info);
                            }
                            else {
                                importInfoFile.internals.push(info);
                            }
                        }
                    }
                    return node;
                }
                catch (e) {
                    e.message += '\n  current node: [' + node.getText().split('\n', 1)[0] + '] infile ' + fileName;
                    throw e;
                }
            }
        };
    };
});
//# sourceMappingURL=transform.js.map